import { Observable } from 'rxjs';
import { IMessageFactory } from './interface/message-factory';
import { IMessageValidator } from './interface/message-validator';
import { IPostmessageAdapter } from './interface/postmessage-adapter';
import { IMessenger, IRequest } from './interface/public-interface';
import { AnyMessage, INotificationObject, IRequestObject, IResponseObject, MappedMessage, MessageType } from './interface/message-objects';
export declare class Messenger implements IMessenger {
    protected readonly messageFactory: IMessageFactory;
    protected readonly messageValidator: IMessageValidator;
    protected readonly adapter: IPostmessageAdapter;
    /**
     * Observable stream of all incoming messages that originate from remoteWindow with a
     * remoteOrigin url matching this.remoteOrigin.
     */
    readonly inboundMessages$: Observable<AnyMessage>;
    /** Filtered subset of inboundMessages$, emitting all messages of type 'notification' */
    readonly notifications$: Observable<INotificationObject>;
    /** Filtered subset of inboundMessages$, emitting all messages of type 'request' */
    readonly requests$: Observable<IRequestObject>;
    /** Filtered subset of inboundMessages$, emitting all messages of type 'response' */
    readonly responses$: Observable<IResponseObject>;
    constructor(messageFactory: IMessageFactory, messageValidator: IMessageValidator, adapter: IPostmessageAdapter);
    request<T = any, U = any>(channel: string, payload?: T): Observable<U>;
    notify<T>(channel: string, payload: T | null): this;
    /**
     * Returns an Observable that emits the subset of inbound notification messages where their channel equals given
     * channel name.
     *
     * @param {string} channel
     * @return {Observable<object>}
     * @public
     */
    requests<T = any, U = any>(channel: string): Observable<IRequest<T, U>>;
    /**
     * Returns an Observable that emits the subset of inbound notification messages where their channel equals given
     * channel name.
     *
     * @param {string} channel
     * @return {Observable<*>}
     * @public
     */
    notifications<T = any>(channel: string): Observable<T>;
    /**
     * Sends a response through given channel to the remote window, carrying given payload.
     *
     * @param {string} requestId
     * @param channel
     * @param {*} payload
     * @return {Messenger}
     * @private
     */
    protected respond<T>(requestId: string, channel: string, payload: T): this;
    /**
     * Creates an Observable that completes after a single emission of the MessageEvent response for request of given
     * requestId. If no matching response is ever received, the Observable never completes.
     *
     * @param {string} requestId
     * @return {Observable<object>}
     * @private
     */
    protected createResponseObservable<T>(requestId: string): Observable<T>;
    /**
     * Returns an Observable emitting all inbound messages` of given type.
     *
     * Returns an Observable emitting a subset of MessageEvent objects emitted
     * by this.inboundMessages$, passing through all MessageEvent objects whose
     * data.type property matches given type.
     *
     * @param {string} type
     * @return {Observable<object>}
     * @private
     */
    protected messagesOfType<T extends MessageType>(type: T): Observable<MappedMessage<T>>;
}
